extern crate wee_alloc;

#[global_allocator]
static ALLOC: wee_alloc::WeeAlloc = wee_alloc::WeeAlloc::INIT;


// webassembly version of all the wordle logic
// because the javascript version is slow as hell on firefox
use std::os::raw::c_char;

static mut LOOKUP: Option<Vec<u8>> = None;
static mut STRINGS: Option<Vec<u8>> = None;
static mut STR_IDXS: Option<Vec<*const c_char>> = None;

#[no_mangle]
pub extern fn init_strings(sz: usize) -> *mut c_char {
    unsafe {
        STRINGS = Some(vec![0u8; sz]);
        STRINGS.as_deref_mut().map(|v| v.as_mut_ptr() as *mut c_char).unwrap()
    }
}

#[no_mangle]
pub extern fn init_idx() -> usize {
    // TODO
    0
}

#[no_mangle]
pub extern fn init_lookup() -> *mut c_char {
    unsafe {
        let num_strings = STR_IDXS.as_ref().map(|idxs| idxs.len()).unwrap_or(0);

        LOOKUP = Some(vec![0u8; num_strings*num_strings]);
        LOOKUP.as_deref_mut().map(|v| v.as_mut_ptr() as *mut c_char).unwrap()
    }
}

#[no_mangle]
pub extern fn calc_match(guess: *const c_char, reference: *const c_char) -> u8 {
    let mut unmatched = vec![0u8; 26];
    let mut ret = 0;
    let mut matched = 0;

    let mut mul = 1;
    for i in 0..5 {
        unsafe {
            if *(guess.offset(i)) == *(reference.offset(i)) {
                ret += 1*mul;
                matched |= 1 << i;
            } else {
                unmatched[*(reference.offset(i)) as usize - 'a' as usize] += 1;
            }
        }
        mul *= 3;
    }

    mul = 1;
    for i in 0..5 {
        if (matched & (1 << i)) == 0 {
            let idx = unsafe { *(guess.offset(i)) as usize - ('a' as usize) };
            if unmatched[idx] > 0 {
                ret += 2*mul;
                unmatched[idx] -= 1;
            }
        }
        mul *= 3;
    }

    ret
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn it_works() {
        let result = add(2, 2);
        assert_eq!(result, 4);
    }
}
